@@ -56,10 +56,10 @@ setAs("dtCMatrix", "denseMatrix", function(from) .Call(Csparse_to_dense, from, -
 
 setAs("dgCMatrix", "vector", function(from) .Call(Csparse_to_vector, from))
 setAs("dsCMatrix", "vector", function(from) .Call(Csparse_to_vector, from))
-setMethod("as.vector", signature(x = "dgCMatrix", mode = "missing"),
-	  function(x, mode) .Call(Csparse_to_vector, x))
-setMethod("as.vector", signature(x = "dsCMatrix", mode = "missing"),
-	  function(x, mode) .Call(Csparse_to_vector, x))
+setMethod("as.vector", "dgCMatrix",
+	  function(x, mode) as.vector(.Call(Csparse_to_vector, x), mode))
+setMethod("as.vector", "dsCMatrix",
+	  function(x, mode) as.vector(.Call(Csparse_to_vector, x), mode))
 ## could do these and more for as(., "numeric") ... but we *do* recommend  as(*,"vector"):
 ## setAs("dgCMatrix", "numeric", Csp2vec)
 ## setAs("dsCMatrix", "numeric", Csp2vec)
@@ -69,13 +69,42 @@ setMethod("as.vector", signature(x = "dsCMatrix", mode = "missing"),
 ## storage so symmetric classes are ok for conversion to matrix.
 ## unit triangular needs special handling
 ##' exported
-.dxC2mat <- function(from, chkUdiag=TRUE) .Call(Csparse_to_matrix, from, chkUdiag, NA)
+.dxC2mat <- function(from, chkUdiag=TRUE)   .Call(Csparse_to_matrix, from, chkUdiag, NA)
 setAs("dgCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from, FALSE, FALSE))
 setAs("dsCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from, FALSE, TRUE))
 setAs("dtCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from, TRUE,  FALSE))
 ## NB: Would *not* be ok for l*Matrix or n*Matrix,
 ## --------- as cholmod coerces to "REAL" aka "double"
 
+..m2dgC <- function(from) .Call(matrix_to_Csparse, from, "dgCMatrix")
+..m2lgC <- function(from) .Call(matrix_to_Csparse, from, "lgCMatrix")
+
+.m2dgC <- function(from) {
+    if(!is.double(from)) storage.mode(from) <- "double"
+    .Call(matrix_to_Csparse, from, "dgCMatrix")
+}
+.m2lgC <- function(from) {
+    if(!is.logical(from)) storage.mode(from) <- "logical"
+    .Call(matrix_to_Csparse, from, "lgCMatrix")
+}
+.m2ngC <- function(from) {
+    if(!is.logical(from)) storage.mode(from) <- "logical"
+    if(anyNA(from)) stop("cannot coerce NA values to pattern \"ngCMatrix\"")
+    .Call(matrix_to_Csparse, from, "ngCMatrix")
+}
+setAs("matrix", "dgCMatrix", .m2dgC)
+setAs("matrix", "lgCMatrix", .m2lgC)
+setAs("matrix", "ngCMatrix", .m2ngC)
+
+## Here, use .m2dgC() instead of ..m2dgC() as C-level
+##  matrix_to_Csparse(x, "dgCMatrix")  fails when x is *integer* :
+setAs("matrix", "CsparseMatrix", ## => choosing 'l*' or 'dgCMatrix' (no tri-, sym-, diag-):
+      function(from) (if(is.logical(from)) ..m2lgC else .m2dgC)(from))
+
+setAs("numeric", "CsparseMatrix",
+      function(from) (if(is.logical(from)) ..m2lgC else .m2dgC)(as.matrix.default(from)))
+
+
 setAs("CsparseMatrix", "symmetricMatrix",
       function(from) {
 	  if(isSymmetric(from)) forceCspSymmetric(from)
@@ -148,9 +177,8 @@ subCsp_ij <- function(x, i, j, drop)
 	    drop(as(r, "matrix"))
 	else {
 	    if(!is.null(n <- names(dn))) names(r@Dimnames) <- n
-	    if(extends((cx <- getClassDef(class(x))),
-		       "symmetricMatrix")) ## TODO? make more efficient:
-		.gC2sym(r, uplo = x@uplo)## preserve uplo !
+	    if(extends((cx <- getClassDef(class(x))), "symmetricMatrix"))
+		.gC2sym(r, uplo = x@uplo) # preserving uplo
 	    else if(extends(cx, "triangularMatrix") && !is.unsorted(ii))
 		as(r, "triangularMatrix")
 	    else r
@@ -272,7 +300,7 @@ replCmat4 <- function(x, i1, i2, iMi, jMi, value, spV = is(value,"sparseVector")
     if(extends(clDx, "dMatrix")) {
 	has.x <- TRUE
 	x <- .Call(dCsparse_subassign,
-		   if(clx %in% c("dgCMatrix", "dtCMatrix"))x
+		   if(clx %in% c("dgCMatrix", "dtCMatrix")) x
 		   else as(x, "dgCMatrix"),
 		   i1, i2,
 		   as(value, "sparseVector"))
@@ -280,7 +308,7 @@ replCmat4 <- function(x, i1, i2, iMi, jMi, value, spV = is(value,"sparseVector")
     else if(extends(clDx, "lMatrix")) {
 	has.x <- TRUE
 	x <- .Call(lCsparse_subassign,
-		   if(clx %in% c("lgCMatrix", "ltCMatrix"))x
+		   if(clx %in% c("lgCMatrix", "ltCMatrix")) x
 		   else as(x, "lgCMatrix"),
 		   i1, i2,
 		   as(value, "sparseVector"))
