@@ -36,7 +36,8 @@ spMatrix <- function(nrow, ncol,
 
 sparseMatrix <- function(i = ep, j = ep, p, x, dims, dimnames,
                          symmetric = FALSE, triangular = FALSE, index1 = TRUE,
-                         giveCsparse = TRUE, check = TRUE, use.last.ij = FALSE)
+                         repr = "C", giveCsparse = (repr == "C"),
+                         check = TRUE, use.last.ij = FALSE)
 {
   ## Purpose: user-level substitute for most  new(<sparseMatrix>, ..) calls
   ## Author: Douglas Bates, Date: 12 Jan 2009, based on Martin's version
@@ -48,6 +49,13 @@ sparseMatrix <- function(i = ep, j = ep, p, x, dims, dimnames,
             stop("'p' must be a non-decreasing vector (0, ...)")
         ep <- rep.int(seq_along(dp), dp)
     }
+    stopifnot(length(repr) == 1L, repr %in% c("C", "T", "R"))
+    ## NB: up to 2020-05, only had giveCsparse=TRUE  --> "C" or "T" -- remain back-compatible:
+    if(missing(repr) && !giveCsparse) {
+	warning("'giveCsparse' has been deprecated; setting 'repr = \"T\"' for you")
+	repr <- "T"
+    } else if(!missing(repr) && !missing(giveCsparse))
+	warning("'giveCsparse' has been deprecated; will use 'repr' instead")
     ## i and j are now both defined (via default = ep).  Make them 1-based indices.
     i1 <- as.logical(index1)[1]
     i <- as.integer(i + !(m.i || i1))
@@ -110,7 +118,11 @@ sparseMatrix <- function(i = ep, j = ep, p, x, dims, dimnames,
     if(!missing(dimnames))
 	r@Dimnames <- .fixupDimnames(dimnames)
     if(check) validObject(r)
-    if(giveCsparse) as(r, "CsparseMatrix") else r
+    switch(repr,
+	   "C" = as(r, "CsparseMatrix"),
+	   "T" =    r,# TsparseMatrix
+	   "R" = as(r, "RsparseMatrix"),
+	   stop("invalid 'repr'; must be \"C\", \"T\", or \"R\""))
 }
 
 ## "graph" coercions -- this needs the graph package which is currently
@@ -370,13 +382,7 @@ setReplaceMethod("[", signature(x = "sparseMatrix", i = "ANY", j = "ANY",
 	if(is.null(dim(cx))) {# e.g. in	1 x 1 case
 	    dim(cx) <- dim(m)
 	    dimnames(cx) <- dn
-	} else ## workaround bug in apply() which has lost row names:
-	    if(getRversion() < "3.2" && !is.null(names(dn))) {
-		if(is.null(dimnames(cx)))
-		    dimnames(cx) <- dn
-		else
-		    names(dimnames(cx)) <- names(dn)
-            }
+	}
     }
     if (missing(col.names))
 	col.names <- {
@@ -416,9 +422,10 @@ formatSparseM <- function(x, zero.print = ".", align = c("fancy", "right"),
 {
     cld <- getClassDef(class(x))
     if(is.null(asLogical)) {
-        binary <- extends(cld,"nsparseMatrix") || extends(cld, "indMatrix")# -> simple T / F
-        asLogical <- { binary || extends(cld,"lsparseMatrix") ||
-                       extends(cld,"matrix") && is.logical(x) }
+	asLogical <- extends1of(cld,
+				c("nsparseMatrix", "indMatrix", # -> simple T / F{ binary
+				  "lsparseMatrix")) ||
+	    (extends(cld, "matrix") && is.logical(x))
 					# has NA and (non-)structural FALSE
     }
     if(missing(cx))
@@ -496,8 +503,8 @@ formatSparseM <- function(x, zero.print = ".", align = c("fancy", "right"),
     cx
 }## formatSparseM()
 
-## utility used inside sparseMatrix print()ing which might be useful
-## outside the Matrix package:
+##' The `format()` method for sparse Matrices; also used inside sparseMatrix print()ing,
+##' exported as it might be useful directly.
 formatSpMatrix <- function(x, digits = NULL, # getOption("digits"),
                            maxp = 1e9, # ~ 1/2 * .Machine$integer.max, ## getOption("max.print"),
                            cld = getClassDef(class(x)), zero.print = ".",
@@ -522,6 +529,7 @@ formatSpMatrix <- function(x, digits = NULL, # getOption("digits"),
 	}
     }
 
+    if(maxp < 100) maxp <- 100L # "stop gap"
     if(prod(d) > maxp) { # "Large" => will be "cut"
         ## only coerce to dense that part which won't be cut :
         nr <- maxp %/% d[2]
@@ -562,7 +570,7 @@ formatSpMatrix <- function(x, digits = NULL, # getOption("digits"),
 ## FIXME: prTriang() in ./Auxiliaries.R  should also get  align = "fancy"
 ##
 printSpMatrix <- function(x, digits = NULL, # getOption("digits"),
-			  maxp = getOption("max.print"),
+			  maxp = max(100L, getOption("max.print")),
 			  cld = getClassDef(class(x)), zero.print = ".",
 			  col.names, note.dropping.colnames = TRUE, uniDiag = TRUE,
 			  col.trailer = '', align = c("fancy", "right"))
@@ -581,7 +589,7 @@ printSpMatrix <- function(x, digits = NULL, # getOption("digits"),
 
 ##' The "real" show() / print() method, calling the above printSpMatrix():
 printSpMatrix2 <- function(x, digits = NULL, # getOption("digits"),
-                           maxp = getOption("max.print"), zero.print = ".",
+                           maxp = max(100L, getOption("max.print")), zero.print = ".",
                            col.names, note.dropping.colnames = TRUE, uniDiag = TRUE,
                            suppRows = NULL, suppCols = NULL,
                            col.trailer = if(suppCols) "......" else "",
@@ -598,11 +606,11 @@ printSpMatrix2 <- function(x, digits = NULL, # getOption("digits"),
     setW <-  !missing(width) && width > getOption("width")
     if(setW) {
 	op <- options(width = width) ; on.exit( options(op) ) }
-    if((identical(suppRows,FALSE) && identical(suppCols, FALSE)) ||
+    if((isFALSE(suppRows) && isFALSE(suppCols)) ||
        (!isTRUE(suppRows) && !isTRUE(suppCols) && prod(d) <= maxp))
     { ## "small matrix" and supp* not TRUE : no rows or columns are suppressed
         if(missing(col.trailer) && is.null(suppCols))
-            suppCols <- FALSE # for 'col.trailer'
+            suppCols <- FALSE # for default 'col.trailer'
         printSpMatrix(x, cld=cld, digits=digits, maxp=maxp,
                       zero.print=zero.print, col.names=col.names,
                       note.dropping.colnames=note.dropping.colnames, uniDiag=uniDiag,
@@ -632,9 +640,12 @@ printSpMatrix2 <- function(x, digits = NULL, # getOption("digits"),
         if(is.null(suppRows)) suppRows <- (nr < d[1])
 	if(suppRows) {
 	    n2 <- ceiling(nr / 2)
+            nr1 <- min(d[1], max(1L, n2)) #{rows} in 1st part
+            nr2 <- max(1L, nr-n2)         #{rows} in 2nd part
+            nr <- nr1+nr2 # total #{rows} to be printed
 	    if(fitWidth) {
 		## one iteration of improving the width, by "fake printing" :
-		cM <- formatSpMatrix(x[seq_len(min(d[1], max(1, n2))), , drop = FALSE],
+		cM <- formatSpMatrix(x[seq_len(nr1), , drop = FALSE],
 				     digits=digits, maxp=maxp, zero.print=zero.print,
 				     col.names=col.names, align=align,
 				     note.dropping.colnames=note.dropping.colnames, uniDiag=FALSE)
@@ -646,16 +657,18 @@ printSpMatrix2 <- function(x, digits = NULL, # getOption("digits"),
                     if(!setW) on.exit( options(op) )
                 }
 	    }
-	    printSpMatrix(x[seq_len(min(d[1], max(1, n2))), , drop=FALSE],
+	    printSpMatrix(x[seq_len(nr1), , drop=FALSE],
 			  digits=digits, maxp=maxp,
 			  zero.print=zero.print, col.names=col.names,
 			  note.dropping.colnames=note.dropping.colnames, uniDiag=uniDiag,
 			  col.trailer = col.trailer, align=align)
-	    suppTxt <- gettext(if(suppCols) "suppressing columns and rows" else "suppressing rows")
+	    suppTxt <- if(suppCols)
+                            gettextf("suppressing %d columns and %d rows", d[2]-nc , d[1]-nr)
+                       else gettextf("suppressing %d rows", d[1]-nr)
 	    cat("\n ..............................",
-		"\n ........", suppTxt, sTxt, "\n", sep='')
+		"\n ........", suppTxt, sTxt, sep='')
 	    ## tail() automagically uses "[..,]" rownames:
-	    printSpMatrix(tail(x, max(1, nr-n2)),
+	    printSpMatrix(tail(x, nr2),
 			  digits=digits, maxp=maxp,
 			  zero.print=zero.print, col.names=col.names,
 			  note.dropping.colnames=note.dropping.colnames, uniDiag=FALSE,
@@ -667,7 +680,7 @@ printSpMatrix2 <- function(x, digits = NULL, # getOption("digits"),
 			  zero.print=zero.print, col.names=col.names,
 			  note.dropping.colnames=note.dropping.colnames, uniDiag=uniDiag,
 			  col.trailer = col.trailer, align=align)
-	    cat("\n .....", gettext("suppressing columns"), sTxt, sep='')
+	    cat("\n .....", gettextf("suppressing %d columns", d[2]-nc), sTxt, sep='')
 	}
 	else stop("logic programming error in printSpMatrix2(), please report")
 
@@ -685,23 +698,28 @@ setMethod("show", signature(object = "sparseMatrix"),
 
 
 ## For very large and very sparse matrices,  the above show()
-## is not really helpful;  Use  summary() as an alternative:
+## is not really helpful;  Use  summary() showing "triplet" as an alternative:
+
+mat2triplet <- function(x, uniqT = FALSE) {
+    T <- as(x, "TsparseMatrix")
+    if(uniqT && anyDuplicatedT(T)) T <- .uniqTsparse(T)
+    if(is(T, "nsparseMatrix"))
+         list(i = T@i + 1L, j = T@j + 1L)
+    else list(i = T@i + 1L, j = T@j + 1L, x = T@x)
+}
 
 setMethod("summary", signature(object = "sparseMatrix"),
-	  function(object, ...) {
-	      d <- dim(object)
-	      T <- as(object, "TsparseMatrix")
-	      ## return a data frame (int, int,	 {double|logical|...})	:
-	      r <- if(is(object,"nsparseMatrix"))
-		  data.frame(i = T@i + 1L, j = T@j + 1L)
-	      else data.frame(i = T@i + 1L, j = T@j + 1L, x = T@x)
-	      attr(r, "header") <-
-		  sprintf('%d x %d sparse Matrix of class "%s", with %d entries',
-			  d[1], d[2], class(object), length(T@i))
-	      ## use ole' S3 technology for such a simple case
-	      class(r) <- c("sparseSummary", class(r))
-	      r
-	  })
+          function(object, uniqT = FALSE, ...) {
+              d <- dim(object)
+              ## return a data frame (int, int,	 {double|logical|...})	:
+              r <- as.data.frame(mat2triplet(object, uniqT=uniqT))
+              attr(r, "header") <-
+                  sprintf('%d x %d sparse Matrix of class "%s", with %d entries',
+                          d[1], d[2], class(object), nrow(r))
+              ## use ole' S3 technology for such a simple case
+              class(r) <- c("sparseSummary", class(r))
+              r
+          })
 
 print.sparseSummary <- function (x, ...) {
     cat(attr(x, "header"),"\n")
