@@ -16,7 +16,8 @@
 	   interactive() && identical(Sys.info()[["user"]], "maechler")) {
 	    ## nothing
 	} else { # ambiguity notices are trashed
-	    options(ambiguousMethodSelection = function(cond) {})
+	    N <- function(cond) NULL; environment(N) <- emptyenv()
+	    options(ambiguousMethodSelection = N)
 	    assign("no.methods.ambiguityNotes", TRUE, envir=.MatrixEnv)
 	}
     }
@@ -59,6 +60,11 @@ if(getRversion() >= "3.2.0") {
     lengths <- function (x, use.names = TRUE) vapply(x, length, 1L, USE.NAMES = use.names)
 }
 
+if(getRversion() < "3.5.0") {
+    isFALSE <- function (x) is.logical(x) && length(x) == 1L && !is.na(x) && !x
+    isTRUE  <- function (x) is.logical(x) && length(x) == 1L && !is.na(x) && x
+}
+
 .onUnload <- function(libpath)
 {
     library.dynam.unload("Matrix", libpath)
