@@ -14,10 +14,13 @@ extern "C" {
 #include <Rdefines.h> /* Rinternals.h + GET_SLOT etc */
 #include <R_ext/RS.h> /* for Memzero() */
 
+/* NB: The "caller" aka "includer" needs to  #include <R_ext/Lapack.h> etc
+ * --  which defines  FCONE  etc (for LTO)
+*/
+
 #define imax2(x, y) ((x < y) ? y : x)
 #define imin2(x, y) ((x < y) ? x : y)
 
-
 // must come after <R.h> above, for clang (2015-08-05)
 #ifdef __GNUC__
 # undef alloca
@@ -47,6 +50,7 @@ extern "C" {
 # endif
 #endif
 
+
 #define Alloca(n, t)   (t *) alloca( (size_t) ( (n) * sizeof(t) ) )
 
 #define SMALL_4_Alloca 10000
@@ -300,10 +304,10 @@ Rboolean any_NA_in_x(SEXP obj)
 static R_INLINE
 SEXP inv_permutation(SEXP p_, SEXP zero_p, SEXP zero_res)
 {
-    int np = 0;
+    int np = 1;
     if(!isInteger(p_)) {p_ = PROTECT(coerceVector(p_, INTSXP)); np++; }
     int *p = INTEGER(p_), n = LENGTH(p_);
-    SEXP val = allocVector(INTSXP, n);// (not PROTECT()ing: no alloc from here on)
+    SEXP val = PROTECT(allocVector(INTSXP, n));
     int *v = INTEGER(val), p_0 = asLogical(zero_p), r_0 = asLogical(zero_res);
     if(!p_0) v--; // ==> use 1-based indices
     // shorter (but not 100% sure if ok: is LHS always eval'ed *before* RHS ?) :
