@@ -1,8 +1,8 @@
 #ifndef MATRIX_DENSE_H
 #define MATRIX_DENSE_H
 
-#include "Rinternals.h"
-#include "R_ext/Lapack.h"
+#include <Rinternals.h>
+#include <R_ext/Lapack.h>
 
 SEXP lsq_dense_Chol(SEXP X, SEXP y);
 SEXP lsq_dense_QR(SEXP X, SEXP y);
